/*
	Physical Simulation in Graphics Scene - master's thesis
    Copyright (C) 2010-2011  Marin Javorka, FIT VUT in Brno, http://www.fit.vutbr.cz, marian.javorka@deq.sk

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
*/
#include "camera.h"

osg::Matrixd FollowCam::getMatrix() const{
	osg::Matrix mat;

	// get the matrix of the object node
	if(_mat->getWorldMatrices().size() <= 0)
		return mat;

	mat = *(_mat->getWorldMatrices().begin());

	osg::Quat rot = mat.getRotate();

	mat.setRotate(osg::Quat(0,0,rot.z(),rot.w()));

	// apply the offset
	mat = osg::Matrixd::inverse(_offset) * mat;

	// return camera position
	return mat;
};

// return inverse matrix
osg::Matrixd FollowCam::getInverseMatrix() const{
	return osg::Matrix::inverse(getMatrix());
};

void FollowCam::changeOffset(osg::Matrixd& offset){
	_offset = offset;	
};
// set matrix is forbidden
void FollowCam::setByMatrix(const osg::Matrixd& m){};
void FollowCam::setByInverseMatrix(const osg::Matrixd& matrix){};

// handler is empty
bool FollowCam::handle(const osgGA::GUIEventAdapter& ea, osgGA::GUIActionAdapter& aa){
	return false;	
};

osg::Matrixd HeliCam::getMatrix() const{
	osg::Matrix mat;

	// get the matrix of the object node
	mat = *(_mat->getWorldMatrices().begin());

	// apply the offset
	mat = osg::Matrixd::inverse(_offset) * mat;

	osg::Vec3f trans = mat.getTrans();

	trans.set(trans.x(),trans.y(),150);

	mat.setRotate(osg::Quat(0,osg::Vec3f(0,0,0)));
	mat.setTrans(trans);

	// return camera position
	return mat;
};

osg::Matrixd DriverCam::getMatrix() const{
	osg::Matrix mat;

	// get the matrix of the object node
	if(_mat->getWorldMatrices().size() <= 0)
		return mat;

	mat = *(_mat->getWorldMatrices().begin());

	// apply the offset
	mat = osg::Matrixd::inverse(_offset) * mat;

	// return camera position
	return mat;
};

osg::Matrixd TVCam::getMatrix() const{
	osg::Matrix mat;

	osg::Matrixd off;
	off.makeLookAt(_eye, _pos, osg::Vec3d(0,0,1));

	// get the matrix of the object node
	if(_mat->getWorldMatrices().size() <= 0)
		return mat;

	mat = *(_mat->getWorldMatrices().begin());

	// apply the offset
	mat = osg::Matrixd::inverse(off) * mat;

	// return camera position
	return mat;
};