/*
	Physical Simulation in Graphics Scene - master's thesis
    Copyright (C) 2010-2011  Marin Javorka, FIT VUT in Brno, http://www.fit.vutbr.cz, marian.javorka@deq.sk

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
*/
#ifndef _CAMERA_H
#define _CAMERA_H

#include <osgGA/MatrixManipulator>
#include <osg/MatrixTransform>

#include <iostream>

class FollowCam: public osgGA::MatrixManipulator {

    public:

		FollowCam(osg::MatrixTransform *node, osg::Matrixd& offset): 
			MatrixManipulator(), _mat(node), _offset(offset)
			{};

        virtual bool handle(const osgGA::GUIEventAdapter& ea, osgGA::GUIActionAdapter& aa);
        virtual void setByMatrix(const osg::Matrixd& m);
        virtual osg::Matrixd getMatrix() const;
        virtual void setByInverseMatrix(const osg::Matrixd &m);
        virtual osg::Matrixd getInverseMatrix() const;

		void changeOffset(osg::Matrixd& offset);

    private:         
        osg::MatrixTransform *_mat;
        osg::Matrixd _offset;
};

class HeliCam: public FollowCam{
	public:

		HeliCam(osg::MatrixTransform *node, osg::Matrixd& offset): 
			FollowCam(node,offset), _mat(node), _offset(offset)
        { };

		virtual osg::Matrixd getMatrix() const;

    private:         

        osg::MatrixTransform *_mat;
        osg::Matrixd _offset;
};

class DriverCam: public FollowCam{
	public:

		DriverCam(osg::MatrixTransform *node, osg::Matrixd& offset, osg::Matrixd off): 
			FollowCam(node,offset), _mat(node), _offset(offset), _off(off)
        { };

		virtual osg::Matrixd getMatrix() const;

    private:         

        osg::MatrixTransform *_mat;
        osg::Matrixd _offset;
		osg::Matrixd _off;
};

class TVCam: public FollowCam{
	public:

		TVCam(osg::MatrixTransform *node, osg::Matrixd& offset, osg::Vec3d& eye, osg::Vec3d pos): 
			FollowCam(node,offset), _mat(node), _offset(offset), _eye(eye), _pos(pos)
        { };

		virtual osg::Matrixd getMatrix() const;
		void changeMT(osg::Vec3d pos){_pos = pos;};

    private:         
        osg::MatrixTransform *_mat;
        osg::Matrixd _offset;
		osg::Vec3d _eye, _pos;
};

#endif