/*
	Physical Simulation in Graphics Scene - master's thesis
    Copyright (C) 2010-2011  Marin Javorka, FIT VUT in Brno, http://www.fit.vutbr.cz, marian.javorka@deq.sk

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
*/
#include "hud.h"

HUD::HUD(const int width, const int height, const float max_rpm, const float max_speed, int player, osg::Vec3d p1col, osg::Vec3d p2col, const std::string gauge_img, const std::string map_img){

	p_num = player;

	scale = width/1920.0f;

	x_offset = 0;

	if(scale < 0.5)
		scale = 0.5;

	map_scale = scale;
	if(player == 1)
		map_scale *= 1.5;

	MAX_RPM = max_rpm;
	MAX_SPEED = max_speed;
	screen_width = width;
	screen_height = height;

	tachometerArrowPAT = new osg::PositionAttitudeTransform;
	speedometerArrowPAT = new osg::PositionAttitudeTransform;
	HUDPAT = new osg::PositionAttitudeTransform;
	mapPAT = new osg::PositionAttitudeTransform;
	timesPAT = new osg::PositionAttitudeTransform;
	P1PAT = new osg::PositionAttitudeTransform;
	P2PAT = new osg::PositionAttitudeTransform;

	HUDPAT->setScale(osg::Vec3d(scale,scale,scale));
	if(player == 0)
		HUDPAT->setPosition(osg::Vec3d(width-512*scale,0,0));
	else
		HUDPAT->setPosition(osg::Vec3d(0,0,0));

	if(player == 0){
		mapPAT->setPosition(osg::Vec3d(0,0,0));
		mapPAT->setScale(osg::Vec3d(map_scale,map_scale,map_scale));
	}
	else{
		mapPAT->setPosition(osg::Vec3d(width-128*map_scale,0,0));
		mapPAT->setScale(osg::Vec3d(map_scale,map_scale,map_scale));
	}

	P1PAT->setScale(osg::Vec3d(map_scale,map_scale,map_scale));
	P2PAT->setScale(osg::Vec3d(map_scale,map_scale,map_scale));
	if(player == 0){
		P1PAT->setPosition(osg::Vec3d(0,0,0));
		P2PAT->setPosition(osg::Vec3d(-128*map_scale,-10000,0));
	}
	else{
		P1PAT->setPosition(osg::Vec3d(width-128*map_scale,0,0));
		P2PAT->setPosition(osg::Vec3d(width-128*map_scale,0,0));
	}

	HUDGear = new osgText::Text();
	HUDInfoCenter = new osgText::Text();
	HUDLapCnt = new osgText::Text();
	HUDBestLap = new osgText::Text();
	HUDLastLap = new osgText::Text();
	HUDRecord = new osgText::Text();

	// HUD geometry node
	HUDGeode = new osg::Group();
	// HUD projection node
	HUDProjectionMatrix = new osg::Projection();

	// set up orthogonal projection
	HUDProjectionMatrix->setMatrix(osg::Matrix::ortho2D(0,width,0,height));

	// set identity as model view matrix
	osg::MatrixTransform* HUDModelViewMatrix = new osg::MatrixTransform;
	HUDModelViewMatrix->setMatrix(osg::Matrix::identity());

	// make sure the model view matrix is not affected by transforms applied above in scene graph
	HUDModelViewMatrix->setReferenceFrame(osg::Transform::ABSOLUTE_RF);

	// add model view matrix as child of the projection matrix
	HUDProjectionMatrix->addChild(HUDModelViewMatrix);
	// add geode as child of model view matrix
	HUDModelViewMatrix->addChild(HUDGeode);

	// create texture
	osg::Texture2D* texture = new osg::Texture2D;
	// load image
	{
		osg::Image* image = osgDB::readImageFile( gauge_img );
		// set image
		texture->setImage(image);
	}
	osg::Texture2D* map_texture = new osg::Texture2D;
	// load image
	{
		osg::Image* image = osgDB::readImageFile( map_img );
		// set image
		map_texture->setImage(image);
	}
	osg::Texture2D* p1_texture = new osg::Texture2D;
	// load image
	{
		osg::Image* image = osgDB::readImageFile( "data/Brno/dot_w.png" );
		// set image
		p1_texture->setImage(image);
	}

	// create texture
	osg::Texture2D* arrowTexture = new osg::Texture2D;
	// load image
	{
		osg::Image* image = osgDB::readImageFile( "data/HUD/arrow.png" );
		// set image
		arrowTexture->setImage(image);
	}

	// create texture
	osg::Texture2D* watchTexture = new osg::Texture2D;
	// load image
	{
		osg::Image* image = osgDB::readImageFile( "data/HUD/watch.png" );
		// set image
		watchTexture->setImage(image);
	}

	////////////////////
	// create map
	{
		osg::Geometry* map = new osg::Geometry();

		// vertices
		osg::Vec3Array* vertices = new osg::Vec3Array;
		vertices->push_back(osg::Vec3(0,0,-1));
		vertices->push_back(osg::Vec3(256,0,-1));
		vertices->push_back(osg::Vec3(256,512,-1));
		vertices->push_back(osg::Vec3(0,512,-1));

		// indices
		osg::DrawElementsUInt* indices = new osg::DrawElementsUInt(osg::PrimitiveSet::POLYGON,0);
		indices->push_back(0);
		indices->push_back(1);
		indices->push_back(2);
		indices->push_back(3);

		// texture coordinates
		osg::Vec2Array* texcoords = new osg::Vec2Array(4);
		(*texcoords)[0].set(0.0f,0.0f);
		(*texcoords)[1].set(1.0f,0.0f);
		(*texcoords)[2].set(1.0f,1.0f);
		(*texcoords)[3].set(0.0f,1.0f);

		// set texture coordinates
		map->setTexCoordArray(0,texcoords);

		osg::Vec3Array* normals = new osg::Vec3Array;
		normals->push_back(osg::Vec3(0.0f,0.0f,1.0f));

		map->setNormalArray(normals);
		map->setNormalBinding(osg::Geometry::BIND_OVERALL);
		map->addPrimitiveSet(indices);
		map->setVertexArray(vertices);

		osg::StateSet* stateSet = new osg::StateSet();
		stateSet->setTextureAttributeAndModes(0, map_texture, osg::StateAttribute::ON);

		// turn on blending
		stateSet->setMode(GL_BLEND, osg::StateAttribute::ON);

		// disable depth_test
		stateSet->setMode(GL_DEPTH_TEST, osg::StateAttribute::OFF);
		stateSet->setRenderingHint(osg::StateSet::TRANSPARENT_BIN);

		// set StateSet
		map->setStateSet(stateSet);

		// add tachometer to HUD geode
		osg::Geode* node = new osg::Geode;
		node->addDrawable(map);
		mapPAT->addChild(node);
		HUDGeode->addChild(mapPAT);
	}

	////////////////////
	// create player dot
	{
		osg::Geometry* dot = new osg::Geometry();

		// vertices
		osg::Vec3Array* vertices = new osg::Vec3Array;
		vertices->push_back(osg::Vec3(-5,-5,0));
		vertices->push_back(osg::Vec3(5,-5,0));
		vertices->push_back(osg::Vec3(5,15,0));
		vertices->push_back(osg::Vec3(-5,15,0));

		// indices
		osg::DrawElementsUInt* indices = new osg::DrawElementsUInt(osg::PrimitiveSet::POLYGON,0);
		indices->push_back(0);
		indices->push_back(1);
		indices->push_back(2);
		indices->push_back(3);

		// texture coordinates
		osg::Vec2Array* texcoords = new osg::Vec2Array(4);
		(*texcoords)[0].set(0.0f,0.0f);
		(*texcoords)[1].set(1.0f,0.0f);
		(*texcoords)[2].set(1.0f,1.0f);
		(*texcoords)[3].set(0.0f,1.0f);

		// set texture coordinates
		dot->setTexCoordArray(0,texcoords);

		osg::Vec4Array* colors = new osg::Vec4Array;
		osg::Vec4 color = osg::Vec4(p1col.x(), p1col.y(), p1col.z(), 1.0f);
		colors->push_back(color);
		colors->push_back(color);
		colors->push_back(color);
		colors->push_back(color);

		dot->setColorArray(colors);
		dot->setColorBinding(osg::Geometry::BIND_PER_VERTEX);

		osg::Vec3Array* normals = new osg::Vec3Array;
		normals->push_back(osg::Vec3(0.0f,0.0f,1.0f));

		dot->setNormalArray(normals);
		dot->setNormalBinding(osg::Geometry::BIND_OVERALL);
		dot->addPrimitiveSet(indices);
		dot->setVertexArray(vertices);

		osg::StateSet* stateSet = new osg::StateSet();
		stateSet->setTextureAttributeAndModes(0, p1_texture, osg::StateAttribute::ON);

		// turn on blending
		stateSet->setMode(GL_BLEND, osg::StateAttribute::ON);

		// disable depth_test
		stateSet->setMode(GL_DEPTH_TEST, osg::StateAttribute::OFF);
		stateSet->setRenderingHint(osg::StateSet::TRANSPARENT_BIN);

		// set StateSet
		dot->setStateSet(stateSet);

		// add tachometer to HUD geode
		osg::Geode* node = new osg::Geode;
		node->addDrawable(dot);
		P1PAT->addChild(node);
		HUDGeode->addChild(P1PAT);
	}

	////////////////////
	// create player dot
	{
		osg::Geometry* dot = new osg::Geometry();

		// vertices
		osg::Vec3Array* vertices = new osg::Vec3Array;
		vertices->push_back(osg::Vec3(-5,-5,0));
		vertices->push_back(osg::Vec3(5,-5,0));
		vertices->push_back(osg::Vec3(5,15,0));
		vertices->push_back(osg::Vec3(-5,15,0));

		// indices
		osg::DrawElementsUInt* indices = new osg::DrawElementsUInt(osg::PrimitiveSet::POLYGON,0);
		indices->push_back(0);
		indices->push_back(1);
		indices->push_back(2);
		indices->push_back(3);

		// texture coordinates
		osg::Vec2Array* texcoords = new osg::Vec2Array(4);
		(*texcoords)[0].set(0.0f,0.0f);
		(*texcoords)[1].set(1.0f,0.0f);
		(*texcoords)[2].set(1.0f,1.0f);
		(*texcoords)[3].set(0.0f,1.0f);

		// set texture coordinates
		dot->setTexCoordArray(0,texcoords);

		osg::Vec4Array* colors = new osg::Vec4Array;
		osg::Vec4 color = osg::Vec4(p2col.x(), p2col.y(), p2col.z(), 1.0f);
		colors->push_back(color);
		colors->push_back(color);
		colors->push_back(color);
		colors->push_back(color);

		dot->setColorArray(colors);
		dot->setColorBinding(osg::Geometry::BIND_PER_VERTEX);

		osg::Vec3Array* normals = new osg::Vec3Array;
		normals->push_back(osg::Vec3(0.0f,0.0f,1.0f));

		dot->setNormalArray(normals);
		dot->setNormalBinding(osg::Geometry::BIND_OVERALL);
		dot->addPrimitiveSet(indices);
		dot->setVertexArray(vertices);

		osg::StateSet* stateSet = new osg::StateSet();
		stateSet->setTextureAttributeAndModes(0, p1_texture, osg::StateAttribute::ON);

		// turn on blending
		stateSet->setMode(GL_BLEND, osg::StateAttribute::ON);

		// disable depth_test
		stateSet->setMode(GL_DEPTH_TEST, osg::StateAttribute::OFF);
		stateSet->setRenderingHint(osg::StateSet::TRANSPARENT_BIN);

		// set StateSet
		dot->setStateSet(stateSet);

		// add tachometer to HUD geode
		osg::Geode* node = new osg::Geode;
		node->addDrawable(dot);
		P2PAT->addChild(node);
		HUDGeode->addChild(P2PAT);
	}

	////////////////////
	// create tachometer
	{
		osg::Geometry* tachometer = new osg::Geometry();

		// vertices
		osg::Vec3Array* vertices = new osg::Vec3Array;
		vertices->push_back(osg::Vec3(0,0,-1));
		vertices->push_back(osg::Vec3(512,0,-1));
		vertices->push_back(osg::Vec3(512,512,-1));
		vertices->push_back(osg::Vec3(0,512,-1));

		// indices
		osg::DrawElementsUInt* indices = new osg::DrawElementsUInt(osg::PrimitiveSet::POLYGON,0);
		indices->push_back(0);
		indices->push_back(1);
		indices->push_back(2);
		indices->push_back(3);

		// texture coordinates
		osg::Vec2Array* texcoords = new osg::Vec2Array(4);
		(*texcoords)[0].set(0.0f,0.0f);
		(*texcoords)[1].set(1.0f,0.0f);
		(*texcoords)[2].set(1.0f,1.0f);
		(*texcoords)[3].set(0.0f,1.0f);

		// set texture coordinates
		tachometer->setTexCoordArray(0,texcoords);

		osg::Vec3Array* normals = new osg::Vec3Array;
		normals->push_back(osg::Vec3(0.0f,0.0f,1.0f));

		//osg::Vec4Array* colors = new osg::Vec4Array;
		//osg::Vec4 color = osg::Vec4(1.0f, 1.0f, 1.0f, 1.0f);
		//colors->push_back(color);
		//colors->push_back(color);
		//colors->push_back(color);
		//colors->push_back(color);

		//tachometer->setColorArray(colors);
		//tachometer->setColorBinding(osg::Geometry::BIND_PER_VERTEX);

		tachometer->setNormalArray(normals);
		tachometer->setNormalBinding(osg::Geometry::BIND_OVERALL);
		tachometer->addPrimitiveSet(indices);
		tachometer->setVertexArray(vertices);

		osg::StateSet* stateSet = new osg::StateSet();
		stateSet->setTextureAttributeAndModes(0, texture, osg::StateAttribute::ON);

		// turn on blending
		stateSet->setMode(GL_BLEND, osg::StateAttribute::ON);

		// disable depth_test
		stateSet->setMode(GL_DEPTH_TEST, osg::StateAttribute::OFF);
		stateSet->setRenderingHint(osg::StateSet::TRANSPARENT_BIN);

		// set StateSet
		tachometer->setStateSet(stateSet);

		// add tachometer to HUD geode
		osg::Geode* node = new osg::Geode;
		node->addDrawable(tachometer);
		HUDPAT->addChild(node);
	}

	////////////////////
	// create speedometer arrow
	{

		osg::Geometry* speedometer_arrow = new osg::Geometry();

		// vertices
		osg::Vec3Array* vertices = new osg::Vec3Array;
		vertices->push_back(osg::Vec3(-192,-24,-1));
		vertices->push_back(osg::Vec3(192,-24,-1));
		vertices->push_back(osg::Vec3(192,24,-1));
		vertices->push_back(osg::Vec3(-192,24,-1));

		// indices
		osg::DrawElementsUInt* indices = new osg::DrawElementsUInt(osg::PrimitiveSet::POLYGON,0);
		indices->push_back(0);
		indices->push_back(1);
		indices->push_back(2);
		indices->push_back(3);

		// texture coordinates
		osg::Vec2Array* texcoords = new osg::Vec2Array(4);
		(*texcoords)[0].set(0.0f,0.0f);
		(*texcoords)[1].set(1.0f,0.0f);
		(*texcoords)[2].set(1.0f,1.0f);
		(*texcoords)[3].set(0.0f,1.0f);

		// set texture coordinates
		speedometer_arrow->setTexCoordArray(0,texcoords);

		osg::Vec3Array* normals = new osg::Vec3Array;
		normals->push_back(osg::Vec3(0.0f,0.0f,1.0f));

		speedometer_arrow->setNormalArray(normals);
		speedometer_arrow->setNormalBinding(osg::Geometry::BIND_OVERALL);
		speedometer_arrow->addPrimitiveSet(indices);
		speedometer_arrow->setVertexArray(vertices);

		osg::StateSet* stateSet = new osg::StateSet();
		stateSet->setTextureAttributeAndModes(0, arrowTexture, osg::StateAttribute::ON);

		// turn on blending
		stateSet->setMode(GL_BLEND, osg::StateAttribute::ON);

		// disable depth_test
		stateSet->setMode(GL_DEPTH_TEST, osg::StateAttribute::OFF);
		stateSet->setRenderingHint(osg::StateSet::TRANSPARENT_BIN);

		// set StateSet
		speedometer_arrow->setStateSet(stateSet);

		// add speedometer to HUD geode
		osg::Geode* speedometer_arrow_geode = new osg::Geode;
		speedometer_arrow_geode->addDrawable(speedometer_arrow);
		speedometerArrowPAT->addChild(speedometer_arrow_geode);
		HUDPAT->addChild(speedometerArrowPAT);
	}

	speedometerArrowPAT->setAttitude(osg::Quat((74+196)*3.14/180,osg::Vec3f(0,0,1)));

	

	////////////////////
	// create tachometer arrow
	{

		osg::Geometry* tachometer_arrow = new osg::Geometry();

		// vertices
		osg::Vec3Array* vertices = new osg::Vec3Array;
		vertices->push_back(osg::Vec3(-141,-17,-1));
		vertices->push_back(osg::Vec3(141,-17,-1));
		vertices->push_back(osg::Vec3(141,17,-1));
		vertices->push_back(osg::Vec3(-141,17,-1));

		// indices
		osg::DrawElementsUInt* indices = new osg::DrawElementsUInt(osg::PrimitiveSet::POLYGON,0);
		indices->push_back(0);
		indices->push_back(1);
		indices->push_back(2);
		indices->push_back(3);

		// texture coordinates
		osg::Vec2Array* texcoords = new osg::Vec2Array(4);
		(*texcoords)[0].set(0.0f,0.0f);
		(*texcoords)[1].set(1.0f,0.0f);
		(*texcoords)[2].set(1.0f,1.0f);
		(*texcoords)[3].set(0.0f,1.0f);

		// set texture coordinates
		tachometer_arrow->setTexCoordArray(0,texcoords);

		osg::Vec3Array* normals = new osg::Vec3Array;
		normals->push_back(osg::Vec3(0.0f,0.0f,1.0f));

		tachometer_arrow->setNormalArray(normals);
		tachometer_arrow->setNormalBinding(osg::Geometry::BIND_OVERALL);
		tachometer_arrow->addPrimitiveSet(indices);
		tachometer_arrow->setVertexArray(vertices);

		osg::StateSet* stateSet = new osg::StateSet();
		stateSet->setTextureAttributeAndModes(0, arrowTexture, osg::StateAttribute::ON);

		// turn on blending
		stateSet->setMode(GL_BLEND, osg::StateAttribute::ON);

		// disable depth_test
		stateSet->setMode(GL_DEPTH_TEST, osg::StateAttribute::OFF);
		stateSet->setRenderingHint(osg::StateSet::TRANSPARENT_BIN);

		// set StateSet
		tachometer_arrow->setStateSet(stateSet);

		// add tachometer to HUD geode
		osg::Geode* tachometer_arrow_geode = new osg::Geode;
		tachometer_arrow_geode->addDrawable(tachometer_arrow);
		tachometerArrowPAT->addChild(tachometer_arrow_geode);
		HUDPAT->addChild(tachometerArrowPAT);
	}

	tachometerArrowPAT->setAttitude(osg::Quat((74+196)*3.14/180,osg::Vec3f(0,0,1)));


	// static texts
	{
		osg::Geode* drawable = new osg::Geode;
		HUDGeode->addChild(drawable);
		{
			osgText::Text* staticText = new osgText::Text();
			staticText->setCharacterSize(20);
			staticText->setFont("data/font/BAUERG__.TTF");
			staticText->setText("LAP:       /--");
			staticText->setAxisAlignment(osgText::Text::SCREEN);
			staticText->setAlignment(osgText::Text::LEFT_BOTTOM);
			staticText->setPosition( osg::Vec3(30,height-50,-1) );
			staticText->setColor( osg::Vec4(1, 1, 1, 0.9) );
			staticText->setBackdropType(osgText::Text::DROP_SHADOW_BOTTOM_RIGHT);
			drawable->addDrawable(staticText);

			// HUDLapCnt
			HUDLapCnt->setCharacterSize(30);
			HUDLapCnt->setFont("data/font/BAUERG__.TTF");
			HUDLapCnt->setText("--");
			HUDLapCnt->setDataVariance(osg::Object::DYNAMIC);
			HUDLapCnt->setAxisAlignment(osgText::Text::SCREEN);
			HUDLapCnt->setAlignment(osgText::Text::RIGHT_CENTER);
			HUDLapCnt->setPosition( osg::Vec3(153,height-40,-1) );
			HUDLapCnt->setColor( osg::Vec4(1, 1, 1, 0.9) );
			HUDLapCnt->setBackdropType(osgText::Text::DROP_SHADOW_BOTTOM_RIGHT);
			drawable->addDrawable(HUDLapCnt);
		}
		{
			osgText::Text* staticText = new osgText::Text();
			staticText->setCharacterSize(20);
			staticText->setFont("data/font/BAUERG__.TTF");
			staticText->setText("BEST LAP:");
			staticText->setAxisAlignment(osgText::Text::SCREEN);
			staticText->setAlignment(osgText::Text::LEFT_BOTTOM);
			staticText->setPosition( osg::Vec3(30,height-90,-1) );
			staticText->setColor( osg::Vec4(1, 1, 1, 0.9) );
			staticText->setBackdropType(osgText::Text::DROP_SHADOW_BOTTOM_RIGHT);
			drawable->addDrawable(staticText);

			// HUDBestLap
			HUDBestLap->setCharacterSize(30);
			HUDBestLap->setFont("data/font/BAUERG__.TTF");
			HUDBestLap->setText("--:--.--");
			HUDBestLap->setDataVariance(osg::Object::DYNAMIC);
			HUDBestLap->setAxisAlignment(osgText::Text::SCREEN);
			HUDBestLap->setAlignment(osgText::Text::LEFT_CENTER);
			HUDBestLap->setPosition( osg::Vec3(190,height-79,-1) );
			HUDBestLap->setColor( osg::Vec4(1, 1, 1, 0.9) );
			HUDBestLap->setBackdropType(osgText::Text::DROP_SHADOW_BOTTOM_RIGHT);
			drawable->addDrawable(HUDBestLap);

			{
				osg::Geometry* sector_time = new osg::Geometry();

				// vertices
				osg::Vec3Array* vertices = new osg::Vec3Array;
				vertices->push_back(osg::Vec3(350,height-92,-2));
				vertices->push_back(osg::Vec3(480,height-92,-2));
				vertices->push_back(osg::Vec3(480,height-72,-2));
				vertices->push_back(osg::Vec3(350,height-72,-2));

				// indices
				osg::DrawElementsUInt* indices = new osg::DrawElementsUInt(osg::PrimitiveSet::POLYGON,0);
				indices->push_back(0);
				indices->push_back(1);
				indices->push_back(2);
				indices->push_back(3);

				// texture coordinates
				osg::Vec2Array* texcoords = new osg::Vec2Array(4);
				(*texcoords)[0].set(0.0f,0.0f);
				(*texcoords)[1].set(1.0f,0.0f);
				(*texcoords)[2].set(1.0f,1.0f);
				(*texcoords)[3].set(0.0f,1.0f);

				// set texture coordinates
				sector_time->setTexCoordArray(0,texcoords);

				osg::Vec3Array* normals = new osg::Vec3Array;
				normals->push_back(osg::Vec3(0.0f,0.0f,1.0f));

				osg::Vec4Array* colors = new osg::Vec4Array;
				osg::Vec4 black = osg::Vec4(0.0f, 0.0f, 0.0f, 0.7f);
				osg::Vec4 gray = osg::Vec4(0.0f, 0.0f, 0.0f, 0.0f);
				colors->push_back(gray);
				colors->push_back(black);
				colors->push_back(black);
				colors->push_back(gray);

				sector_time->setColorArray(colors);
				sector_time->setColorBinding(osg::Geometry::BIND_PER_VERTEX);

				sector_time->setNormalArray(normals);
				sector_time->setNormalBinding(osg::Geometry::BIND_OVERALL);
				sector_time->addPrimitiveSet(indices);
				sector_time->setVertexArray(vertices);

				osg::StateSet* stateSet = new osg::StateSet();

				// turn on blending
				stateSet->setMode(GL_BLEND, osg::StateAttribute::ON);

				// set StateSet
				sector_time->setStateSet(stateSet);

				drawable->addDrawable(sector_time);
			}

			{
				osg::Geometry* sector_time = new osg::Geometry();

				// vertices
				osg::Vec3Array* vertices = new osg::Vec3Array;
				vertices->push_back(osg::Vec3(480,height-92,-2));
				vertices->push_back(osg::Vec3(580,height-92,-2));
				vertices->push_back(osg::Vec3(580,height-72,-2));
				vertices->push_back(osg::Vec3(480,height-72,-2));

				// indices
				osg::DrawElementsUInt* indices = new osg::DrawElementsUInt(osg::PrimitiveSet::POLYGON,0);
				indices->push_back(0);
				indices->push_back(1);
				indices->push_back(2);
				indices->push_back(3);

				// texture coordinates
				osg::Vec2Array* texcoords = new osg::Vec2Array(4);
				(*texcoords)[0].set(0.0f,0.0f);
				(*texcoords)[1].set(1.0f,0.0f);
				(*texcoords)[2].set(1.0f,1.0f);
				(*texcoords)[3].set(0.0f,1.0f);

				// set texture coordinates
				sector_time->setTexCoordArray(0,texcoords);

				osg::Vec3Array* normals = new osg::Vec3Array;
				normals->push_back(osg::Vec3(0.0f,0.0f,1.0f));

				osg::Vec4Array* colors = new osg::Vec4Array;
				osg::Vec4 orange = osg::Vec4(0.91f, 0.64f, 0.01f, 0.7f);
				colors->push_back(orange);
				colors->push_back(orange);
				colors->push_back(orange);
				colors->push_back(orange);

				sector_time->setColorArray(colors);
				sector_time->setColorBinding(osg::Geometry::BIND_PER_VERTEX);

				sector_time->setNormalArray(normals);
				sector_time->setNormalBinding(osg::Geometry::BIND_OVERALL);
				sector_time->addPrimitiveSet(indices);
				sector_time->setVertexArray(vertices);

				osg::StateSet* stateSet = new osg::StateSet();

				// turn on blending
				stateSet->setMode(GL_BLEND, osg::StateAttribute::ON);

				// set StateSet
				sector_time->setStateSet(stateSet);

				sector_timeP = new osg::Geode();
				sector_timeP->addDrawable(sector_time);
				sector_timeP->setNodeMask(0x0);
				HUDGeode->addChild(sector_timeP);
			}
			{
				osg::Geometry* sector_time = new osg::Geometry();

				// vertices
				osg::Vec3Array* vertices = new osg::Vec3Array;
				vertices->push_back(osg::Vec3(480,height-92,-2));
				vertices->push_back(osg::Vec3(580,height-92,-2));
				vertices->push_back(osg::Vec3(580,height-72,-2));
				vertices->push_back(osg::Vec3(480,height-72,-2));

				// indices
				osg::DrawElementsUInt* indices = new osg::DrawElementsUInt(osg::PrimitiveSet::POLYGON,0);
				indices->push_back(0);
				indices->push_back(1);
				indices->push_back(2);
				indices->push_back(3);

				// texture coordinates
				osg::Vec2Array* texcoords = new osg::Vec2Array(4);
				(*texcoords)[0].set(0.0f,0.0f);
				(*texcoords)[1].set(1.0f,0.0f);
				(*texcoords)[2].set(1.0f,1.0f);
				(*texcoords)[3].set(0.0f,1.0f);

				// set texture coordinates
				sector_time->setTexCoordArray(0,texcoords);

				osg::Vec3Array* normals = new osg::Vec3Array;
				normals->push_back(osg::Vec3(0.0f,0.0f,1.0f));

				osg::Vec4Array* colors = new osg::Vec4Array;
				osg::Vec4 green = osg::Vec4(0.14f, 0.54f, 0.0f, 0.7f);
				colors->push_back(green);
				colors->push_back(green);
				colors->push_back(green);
				colors->push_back(green);

				sector_time->setColorArray(colors);
				sector_time->setColorBinding(osg::Geometry::BIND_PER_VERTEX);

				sector_time->setNormalArray(normals);
				sector_time->setNormalBinding(osg::Geometry::BIND_OVERALL);
				sector_time->addPrimitiveSet(indices);
				sector_time->setVertexArray(vertices);

				osg::StateSet* stateSet = new osg::StateSet();

				// turn on blending
				stateSet->setMode(GL_BLEND, osg::StateAttribute::ON);

				// set StateSet
				sector_time->setStateSet(stateSet);

				sector_timeN = new osg::Geode();
				sector_timeN->addDrawable(sector_time);
				sector_timeN->setNodeMask(0x0);
				HUDGeode->addChild(sector_timeN);
			}

			// HUDBestLapS1
			HUDBestLapS1 = new osgText::Text();
			HUDBestLapS1->setCharacterSize(10);
			HUDBestLapS1->setFont("data/font/BAUERG__.TTF");
			HUDBestLapS1->setText("--:--.---");
			HUDBestLapS1->setDataVariance(osg::Object::DYNAMIC);
			HUDBestLapS1->setAxisAlignment(osgText::Text::SCREEN);
			HUDBestLapS1->setAlignment(osgText::Text::RIGHT_BOTTOM);
			HUDBestLapS1->setPosition( osg::Vec3(475,height-87,-1) );
			HUDBestLapS1->setColor( osg::Vec4(1, 1, 1, 0.9) );
			HUDBestLapS1->setBackdropType(osgText::Text::DROP_SHADOW_BOTTOM_RIGHT);
			drawable->addDrawable(HUDBestLapS1);
			// HUDBestLapS2
			HUDBestLapS2 = new osgText::Text();
			HUDBestLapS2->setCharacterSize(10);
			HUDBestLapS2->setFont("data/font/BAUERG__.TTF");
			HUDBestLapS2->setText("");
			HUDBestLapS2->setDataVariance(osg::Object::DYNAMIC);
			HUDBestLapS2->setAxisAlignment(osgText::Text::SCREEN);
			HUDBestLapS2->setAlignment(osgText::Text::LEFT_BOTTOM);
			HUDBestLapS2->setPosition( osg::Vec3(485,height-87,-1) );
			HUDBestLapS2->setColor( osg::Vec4(1, 1, 1, 0.9) );
			HUDBestLapS2->setBackdropType(osgText::Text::DROP_SHADOW_BOTTOM_RIGHT);
			drawable->addDrawable(HUDBestLapS2);
		}
		{
			osgText::Text* staticText = new osgText::Text();
			staticText->setCharacterSize(20);
			staticText->setFont("data/font/BAUERG__.TTF");
			staticText->setText("LAST LAP:");
			staticText->setAxisAlignment(osgText::Text::SCREEN);
			staticText->setAlignment(osgText::Text::LEFT_BOTTOM);
			staticText->setPosition( osg::Vec3(30,height-130,-1) );
			staticText->setColor( osg::Vec4(1, 1, 1, 0.9) );
			staticText->setBackdropType(osgText::Text::DROP_SHADOW_BOTTOM_RIGHT);
			drawable->addDrawable(staticText);

			// HUDLastLap
			HUDLastLap->setCharacterSize(30);
			HUDLastLap->setFont("data/font/BAUERG__.TTF");
			HUDLastLap->setText("--:--.--");
			HUDLastLap->setDataVariance(osg::Object::DYNAMIC);
			HUDLastLap->setAxisAlignment(osgText::Text::SCREEN);
			HUDLastLap->setAlignment(osgText::Text::LEFT_CENTER);
			HUDLastLap->setPosition( osg::Vec3(190,height-119,-1) );
			HUDLastLap->setColor( osg::Vec4(1, 1, 1, 0.9) );
			HUDLastLap->setBackdropType(osgText::Text::DROP_SHADOW_BOTTOM_RIGHT);
			drawable->addDrawable(HUDLastLap);
		}
		{
			osgText::Text* staticText = new osgText::Text();
			staticText->setCharacterSize(20);
			staticText->setFont("data/font/BAUERG__.TTF");
			staticText->setText("TRACK RECORD:");
			staticText->setAxisAlignment(osgText::Text::SCREEN);
			staticText->setAlignment(osgText::Text::LEFT_BOTTOM);
			staticText->setPosition( osg::Vec3(30,height-170,-1) );
			staticText->setColor( osg::Vec4(1, 1, 1, 0.9) );
			staticText->setBackdropType(osgText::Text::DROP_SHADOW_BOTTOM_RIGHT);
			drawable->addDrawable(staticText);

			// HUDRecordman
			HUDRecordman = new osgText::Text();
			HUDRecordman->setCharacterSize(15);
			HUDRecordman->setFont("data/font/BAUERG__.TTF");
			HUDRecordman->setText("unknown");
			HUDRecordman->setDataVariance(osg::Object::DYNAMIC);
			HUDRecordman->setAxisAlignment(osgText::Text::SCREEN);
			HUDRecordman->setAlignment(osgText::Text::LEFT_CENTER);
			HUDRecordman->setPosition( osg::Vec3(100,height-185,-1) );
			HUDRecordman->setColor( osg::Vec4(1, 1, 1, 0.9) );
			HUDRecordman->setBackdropType(osgText::Text::DROP_SHADOW_BOTTOM_RIGHT);
			drawable->addDrawable(HUDRecordman);
			// HUDRecord
			HUDRecord->setCharacterSize(15);
			HUDRecord->setFont("data/font/BAUERG__.TTF");
			HUDRecord->setText("--:--.--");
			HUDRecord->setDataVariance(osg::Object::DYNAMIC);
			HUDRecord->setAxisAlignment(osgText::Text::SCREEN);
			HUDRecord->setAlignment(osgText::Text::LEFT_TOP);
			HUDRecord->setPosition( osg::Vec3(100,height-200,-1) );
			HUDRecord->setColor( osg::Vec4(1, 1, 1, 0.9) );
			HUDRecord->setBackdropType(osgText::Text::DROP_SHADOW_BOTTOM_RIGHT);
			drawable->addDrawable(HUDRecord);

			{
				osg::Geometry* sector_time = new osg::Geometry();

				// vertices
				osg::Vec3Array* vertices = new osg::Vec3Array;
				vertices->push_back(osg::Vec3(184,height-216,-2));
				vertices->push_back(osg::Vec3(314,height-216,-2));
				vertices->push_back(osg::Vec3(314,height-196,-2));
				vertices->push_back(osg::Vec3(184,height-196,-2));

				// indices
				osg::DrawElementsUInt* indices = new osg::DrawElementsUInt(osg::PrimitiveSet::POLYGON,0);
				indices->push_back(0);
				indices->push_back(1);
				indices->push_back(2);
				indices->push_back(3);

				// texture coordinates
				osg::Vec2Array* texcoords = new osg::Vec2Array(4);
				(*texcoords)[0].set(0.0f,0.0f);
				(*texcoords)[1].set(1.0f,0.0f);
				(*texcoords)[2].set(1.0f,1.0f);
				(*texcoords)[3].set(0.0f,1.0f);

				// set texture coordinates
				sector_time->setTexCoordArray(0,texcoords);

				osg::Vec3Array* normals = new osg::Vec3Array;
				normals->push_back(osg::Vec3(0.0f,0.0f,1.0f));

				osg::Vec4Array* colors = new osg::Vec4Array;
				osg::Vec4 black = osg::Vec4(0.0f, 0.0f, 0.0f, 0.7f);
				osg::Vec4 gray = osg::Vec4(0.0f, 0.0f, 0.0f, 0.0f);
				colors->push_back(gray);
				colors->push_back(black);
				colors->push_back(black);
				colors->push_back(gray);

				sector_time->setColorArray(colors);
				sector_time->setColorBinding(osg::Geometry::BIND_PER_VERTEX);

				sector_time->setNormalArray(normals);
				sector_time->setNormalBinding(osg::Geometry::BIND_OVERALL);
				sector_time->addPrimitiveSet(indices);
				sector_time->setVertexArray(vertices);

				osg::StateSet* stateSet = new osg::StateSet();

				// turn on blending
				stateSet->setMode(GL_BLEND, osg::StateAttribute::ON);

				// set StateSet
				sector_time->setStateSet(stateSet);

				drawable->addDrawable(sector_time);
			}

			{
				osg::Geometry* sector_time = new osg::Geometry();

				// vertices
				osg::Vec3Array* vertices = new osg::Vec3Array;
				vertices->push_back(osg::Vec3(314,height-216,-2));
				vertices->push_back(osg::Vec3(414,height-216,-2));
				vertices->push_back(osg::Vec3(414,height-196,-2));
				vertices->push_back(osg::Vec3(314,height-196,-2));

				// indices
				osg::DrawElementsUInt* indices = new osg::DrawElementsUInt(osg::PrimitiveSet::POLYGON,0);
				indices->push_back(0);
				indices->push_back(1);
				indices->push_back(2);
				indices->push_back(3);

				// texture coordinates
				osg::Vec2Array* texcoords = new osg::Vec2Array(4);
				(*texcoords)[0].set(0.0f,0.0f);
				(*texcoords)[1].set(1.0f,0.0f);
				(*texcoords)[2].set(1.0f,1.0f);
				(*texcoords)[3].set(0.0f,1.0f);

				// set texture coordinates
				sector_time->setTexCoordArray(0,texcoords);

				osg::Vec3Array* normals = new osg::Vec3Array;
				normals->push_back(osg::Vec3(0.0f,0.0f,1.0f));

				osg::Vec4Array* colors = new osg::Vec4Array;
				osg::Vec4 orange = osg::Vec4(0.91f, 0.64f, 0.01f, 0.7f);
				colors->push_back(orange);
				colors->push_back(orange);
				colors->push_back(orange);
				colors->push_back(orange);

				sector_time->setColorArray(colors);
				sector_time->setColorBinding(osg::Geometry::BIND_PER_VERTEX);

				sector_time->setNormalArray(normals);
				sector_time->setNormalBinding(osg::Geometry::BIND_OVERALL);
				sector_time->addPrimitiveSet(indices);
				sector_time->setVertexArray(vertices);

				osg::StateSet* stateSet = new osg::StateSet();

				// turn on blending
				stateSet->setMode(GL_BLEND, osg::StateAttribute::ON);

				// set StateSet
				sector_time->setStateSet(stateSet);

				Recordsector_timeP = new osg::Geode();
				Recordsector_timeP->addDrawable(sector_time);
				Recordsector_timeP->setNodeMask(0x0);
				HUDGeode->addChild(Recordsector_timeP);
			}
			{
				osg::Geometry* sector_time = new osg::Geometry();

				// vertices
				osg::Vec3Array* vertices = new osg::Vec3Array;
				vertices->push_back(osg::Vec3(314,height-216,-2));
				vertices->push_back(osg::Vec3(414,height-216,-2));
				vertices->push_back(osg::Vec3(414,height-196,-2));
				vertices->push_back(osg::Vec3(314,height-196,-2));

				// indices
				osg::DrawElementsUInt* indices = new osg::DrawElementsUInt(osg::PrimitiveSet::POLYGON,0);
				indices->push_back(0);
				indices->push_back(1);
				indices->push_back(2);
				indices->push_back(3);

				// texture coordinates
				osg::Vec2Array* texcoords = new osg::Vec2Array(4);
				(*texcoords)[0].set(0.0f,0.0f);
				(*texcoords)[1].set(1.0f,0.0f);
				(*texcoords)[2].set(1.0f,1.0f);
				(*texcoords)[3].set(0.0f,1.0f);

				// set texture coordinates
				sector_time->setTexCoordArray(0,texcoords);

				osg::Vec3Array* normals = new osg::Vec3Array;
				normals->push_back(osg::Vec3(0.0f,0.0f,1.0f));

				osg::Vec4Array* colors = new osg::Vec4Array;
				osg::Vec4 green = osg::Vec4(0.14f, 0.54f, 0.0f, 0.7f);
				colors->push_back(green);
				colors->push_back(green);
				colors->push_back(green);
				colors->push_back(green);

				sector_time->setColorArray(colors);
				sector_time->setColorBinding(osg::Geometry::BIND_PER_VERTEX);

				sector_time->setNormalArray(normals);
				sector_time->setNormalBinding(osg::Geometry::BIND_OVERALL);
				sector_time->addPrimitiveSet(indices);
				sector_time->setVertexArray(vertices);

				osg::StateSet* stateSet = new osg::StateSet();

				// turn on blending
				stateSet->setMode(GL_BLEND, osg::StateAttribute::ON);

				// set StateSet
				sector_time->setStateSet(stateSet);

				Recordsector_timeN = new osg::Geode();
				Recordsector_timeN->addDrawable(sector_time);
				Recordsector_timeN->setNodeMask(0x0);
				HUDGeode->addChild(Recordsector_timeN);
			}

			// HUDRecordS1
			HUDRecordS1 = new osgText::Text();
			HUDRecordS1->setCharacterSize(10);
			HUDRecordS1->setFont("data/font/BAUERG__.TTF");
			HUDRecordS1->setText("--:--.---");
			HUDRecordS1->setDataVariance(osg::Object::DYNAMIC);
			HUDRecordS1->setAxisAlignment(osgText::Text::SCREEN);
			HUDRecordS1->setAlignment(osgText::Text::RIGHT_BOTTOM);
			HUDRecordS1->setPosition( osg::Vec3(309,height-211,-1) );
			HUDRecordS1->setColor( osg::Vec4(1, 1, 1, 0.9) );
			HUDRecordS1->setBackdropType(osgText::Text::DROP_SHADOW_BOTTOM_RIGHT);
			drawable->addDrawable(HUDRecordS1);
			// HUDRecordS2
			HUDRecordS2 = new osgText::Text();
			HUDRecordS2->setCharacterSize(10);
			HUDRecordS2->setFont("data/font/BAUERG__.TTF");
			HUDRecordS2->setText("");
			HUDRecordS2->setDataVariance(osg::Object::DYNAMIC);
			HUDRecordS2->setAxisAlignment(osgText::Text::SCREEN);
			HUDRecordS2->setAlignment(osgText::Text::LEFT_BOTTOM);
			HUDRecordS2->setPosition( osg::Vec3(319,height-211,-1) );
			HUDRecordS2->setColor( osg::Vec4(1, 1, 1, 0.9) );
			HUDRecordS2->setBackdropType(osgText::Text::DROP_SHADOW_BOTTOM_RIGHT);
			drawable->addDrawable(HUDRecordS2);
		}
	}


	////////////////////
	// create watch
	{
		osg::Geometry* map = new osg::Geometry();

		// vertices
		osg::Vec3Array* vertices = new osg::Vec3Array;
		vertices->push_back(osg::Vec3(width/2-80,height-253,-1));
		vertices->push_back(osg::Vec3(width/2-60,height-253,-1));
		vertices->push_back(osg::Vec3(width/2-60,height-233,-1));
		vertices->push_back(osg::Vec3(width/2-80,height-233,-1));

		// indices
		osg::DrawElementsUInt* indices = new osg::DrawElementsUInt(osg::PrimitiveSet::POLYGON,0);
		indices->push_back(0);
		indices->push_back(1);
		indices->push_back(2);
		indices->push_back(3);

		// texture coordinates
		osg::Vec2Array* texcoords = new osg::Vec2Array(4);
		(*texcoords)[0].set(0.0f,0.0f);
		(*texcoords)[1].set(1.0f,0.0f);
		(*texcoords)[2].set(1.0f,1.0f);
		(*texcoords)[3].set(0.0f,1.0f);

		// set texture coordinates
		map->setTexCoordArray(0,texcoords);

		osg::Vec3Array* normals = new osg::Vec3Array;
		normals->push_back(osg::Vec3(0.0f,0.0f,1.0f));

		map->setNormalArray(normals);
		map->setNormalBinding(osg::Geometry::BIND_OVERALL);
		map->addPrimitiveSet(indices);
		map->setVertexArray(vertices);

		osg::StateSet* stateSet = new osg::StateSet();
		stateSet->setTextureAttributeAndModes(0, watchTexture, osg::StateAttribute::ON);

		// turn on blending
		stateSet->setMode(GL_BLEND, osg::StateAttribute::ON);

		// disable depth_test
		stateSet->setMode(GL_DEPTH_TEST, osg::StateAttribute::OFF);
		stateSet->setRenderingHint(osg::StateSet::TRANSPARENT_BIN);

		// set StateSet
		map->setStateSet(stateSet);

		// add tachometer to HUD geode
		osg::Geode* node = new osg::Geode;
		node->addDrawable(map);
		HUDGeode->addChild(node);
	}

	// actual gear
	HUDGear->setCharacterSize(40*scale);
	HUDGear->setFont("data/font/digital-7.ttf");
	HUDGear->setText("N");
	HUDGear->setAxisAlignment(osgText::Text::SCREEN);
	HUDGear->setAlignment(osgText::Text::CENTER_BOTTOM);
	HUDGear->setPosition( osg::Vec3(203,355,-1) );
	HUDGear->setColor( osg::Vec4(1, 1, 1, 0.9) );
	HUDGear->setDataVariance(osg::Object::DYNAMIC);

	osg::Geode* gearDrawable = new osg::Geode;
	gearDrawable->addDrawable(HUDGear);

	HUDPAT->addChild(gearDrawable);

	HUDInfoCenterTO = DEFAULT_TO;
	HUDInfoCenterStart = 0;

	HUDSectorTimeTO = DEFAULT_TO;
	HUDSectorTimeStart = 0;

	// centered info text
	HUDInfoCenter->setCharacterSize(DEFAULT_FONT);
	HUDInfoCenter->setFont("data/font/AlphaFridgeMagnets.ttf");
	HUDInfoCenter->setAxisAlignment(osgText::Text::SCREEN);
	HUDInfoCenter->setAlignment(osgText::Text::CENTER_TOP);

	HUDInfoCenter->setPosition( osg::Vec3(width/2,height,-0.1) );
	HUDInfoCenter->setColor( osg::Vec4(1, 1, 1, 1) );
	HUDInfoCenter->setDataVariance(osg::Object::DYNAMIC);

	osg::Geode* fullScreenDrawable = new osg::Geode;
	fullScreenDrawable->addDrawable(HUDInfoCenter);

	// centered lap time
	HUDactTime = new osgText::Text();
	HUDactTime->setCharacterSize(15);
	HUDactTime->setFont("data/font/BAUERG__.TTF");
	HUDactTime->setText("00:00.000");
	HUDactTime->setAxisAlignment(osgText::Text::SCREEN);
	HUDactTime->setAlignment(osgText::Text::LEFT_BOTTOM);

	HUDactTime->setPosition( osg::Vec3(width/2-50,height-250,-0.1) );
	HUDactTime->setColor( osg::Vec4(1, 1, 1, 1) );
	HUDactTime->setDataVariance(osg::Object::DYNAMIC);
	fullScreenDrawable->addDrawable(HUDactTime);

	HUDGeode->addChild(HUDPAT);
	HUDGeode->addChild(fullScreenDrawable);

	osg::StateSet* stateSet = new osg::StateSet();

	// disable depth_test
	stateSet->setMode(GL_DEPTH_TEST, osg::StateAttribute::OFF);
	stateSet->setMode(GL_LIGHTING, osg::StateAttribute::OFF);
	
	stateSet->setRenderBinDetails( 11, "RenderBin");

	// set StateSet
	HUDGeode->setStateSet(stateSet);
}

osg::Projection* HUD::getHUD(){
	return HUDProjectionMatrix;
}

void HUD::updateTachometer(VehicleHUDData data){
	tachometerArrowPAT->setPosition(osg::Vec3d(0,0,0));
	tachometerArrowPAT->setAttitude(osg::Quat((90+180*((MAX_RPM-data.RPM)/MAX_RPM))*3.14/180,osg::Vec3f(0,0,1)));
	tachometerArrowPAT->setPosition(osg::Vec3d(146,369,0));

	speedometerArrowPAT->setPosition(osg::Vec3d(0,0,0));
	speedometerArrowPAT->setAttitude(osg::Quat((90+180*((MAX_SPEED-abs(data.speed))/MAX_SPEED))*3.14/180,osg::Vec3f(0,0,1)));
	speedometerArrowPAT->setPosition(osg::Vec3d(322,190,0));

	osgText::String string;
	string.push_back(data.gear);
	HUDGear->setText(string);

	if(HUDInfoCenterStart != 0 && osg::Timer::instance()->delta_m(HUDInfoCenterStart, osg::Timer::instance()->tick()) > HUDInfoCenterTO){
		HUDInfoCenterStart = 0;

		// reset info text properties
		HUDInfoCenter->setText("");
		HUDInfoCenterTO = DEFAULT_TO;
		HUDInfoCenter->setCharacterSize(DEFAULT_FONT);
		HUDInfoCenter->setColor( osg::Vec4(1, 1, 1, 1) );
	}

	if(HUDSectorTimeStart != 0 && osg::Timer::instance()->delta_m(HUDSectorTimeStart, osg::Timer::instance()->tick()) > HUDSectorTimeTO){
		HUDSectorTimeStart = 0;

		// reset info text properties
		HUDBestLapS2->setText("");
		HUDSectorTimeTO = DEFAULT_TO;
		sector_timeP->setNodeMask(0x0);
		sector_timeN->setNodeMask(0x0);
	}

	if(HUDRecordSectorTimeStart != 0 && osg::Timer::instance()->delta_m(HUDRecordSectorTimeStart, osg::Timer::instance()->tick()) > HUDRecordSectorTimeTO){
		HUDRecordSectorTimeStart = 0;

		// reset info text properties
		HUDRecordS2->setText("");
		HUDRecordSectorTimeTO = DEFAULT_TO;
		Recordsector_timeP->setNodeMask(0x0);
		Recordsector_timeN->setNodeMask(0x0);
	}

	if( p_num == 0 ){
		P1PAT->setPosition(osg::Vec3d(0,0,0));
		P1PAT->setAttitude(osg::Quat(0,0,data.pos.z(),data.pos.w()));
		P1PAT->setPosition(osg::Vec3d(x_offset+(data.pos.x()*0.21+38)*map_scale,(data.pos.y()*0.21+328)*map_scale,0));
	}
	else{
		P2PAT->setPosition(osg::Vec3d(0,0,0));
		P2PAT->setAttitude(osg::Quat(0,0,data.pos.z(),data.pos.w()));
		P2PAT->setPosition(osg::Vec3d(screen_width-128*map_scale+(data.pos.x()*0.21+38)*map_scale,(data.pos.y()*0.21+328)*map_scale,0));
	}
}

void HUD::updateMap(osg::Vec4d pos){
	if(p_num == 0){
		P2PAT->setPosition(osg::Vec3d(0,0,0));
		P2PAT->setAttitude(osg::Quat(0,0,pos.z(),pos.w()));
		P2PAT->setPosition(osg::Vec3d(-128*map_scale+(pos.x()*0.21+38)*map_scale,(pos.y()*0.21+328)*map_scale,0));
	}
	else{
		P1PAT->setPosition(osg::Vec3d(0,0,0));
		P1PAT->setAttitude(osg::Quat(0,0,pos.z(),pos.w()));
		P1PAT->setPosition(osg::Vec3d(screen_width-128*map_scale+(pos.x()*0.21+38)*map_scale,(pos.y()*0.21+328)*map_scale,0));
	}
}

void HUD::updateMap(){
	if(p_num == 0){
		map_scale *= 1.5;
		mapPAT->setScale(osg::Vec3d(map_scale,map_scale,map_scale));
		mapPAT->setPosition(osg::Vec3d(-128*map_scale,0,0));
		P1PAT->setPosition(osg::Vec3d(-128*map_scale,0,0));
		P1PAT->setScale(osg::Vec3d(map_scale,map_scale,map_scale));
		P2PAT->setScale(osg::Vec3d(map_scale,map_scale,map_scale));
		x_offset = -128*map_scale;
	}	
}

void HUD::setMainInfoText(std::string text, int timeout, float font_size, osg::Vec4 color){
	if(timeout > 0){
		HUDInfoCenterTO = timeout;
		HUDInfoCenterStart = osg::Timer::instance()->tick();
	}
	else
		HUDInfoCenterStart = 0;

	HUDInfoCenter->setText(text.c_str());
	HUDInfoCenter->setCharacterSize(font_size);
	HUDInfoCenter->setColor( color );
}

void HUD::setSectorTime(std::string text, int timeout, bool positive){
	if(timeout > 0){
		HUDSectorTimeTO = timeout;
		HUDSectorTimeStart = osg::Timer::instance()->tick();
	}
	else
		HUDSectorTimeStart = 0;

	HUDBestLapS2->setText(text.c_str());
	if(positive == true)
		sector_timeP->setNodeMask(0xffffffff);
	else
		sector_timeN->setNodeMask(0xffffffff);
}

void HUD::setRecordSectorTime(std::string text, int timeout, bool positive){
	if(timeout > 0){
		HUDRecordSectorTimeTO = timeout;
		HUDRecordSectorTimeStart = osg::Timer::instance()->tick();
	}
	else
		HUDRecordSectorTimeStart = 0;

	HUDRecordS2->setText(text.c_str());
	if(positive == true)
		Recordsector_timeP->setNodeMask(0xffffffff);
	else
		Recordsector_timeN->setNodeMask(0xffffffff);
}

void HUD::resetMainInfoText(){		
	HUDInfoCenter->setText("");
	HUDInfoCenterStart = 0;

	HUDInfoCenterTO = DEFAULT_TO;
	HUDInfoCenter->setCharacterSize(DEFAULT_FONT);
	HUDInfoCenter->setColor( osg::Vec4(1, 1, 1, 1) );
}