/*
	Physical Simulation in Graphics Scene - master's thesis
    Copyright (C) 2010-2011  Marin Javorka, FIT VUT in Brno, http://www.fit.vutbr.cz, marian.javorka@deq.sk

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
*/
#ifndef HUD_H
#define HUD_H

#include <osgText/Text>
#include <osg/MatrixTransform>
#include <osg/PositionAttitudeTransform>
#include <osg/Projection>
#include <osg/Group>
#include <osg/Node>
#include <osg/Geode>
#include <osgDB/ReadFile>
#include <vector>

#include "vehicle.h"

#define DEFAULT_FONT 40
#define DEFAULT_TO 3000

using namespace std;

class HUD{
	private:
		osg::Group* HUDGeode;
		osg::Projection* HUDProjectionMatrix;
		osg::ref_ptr<osg::PositionAttitudeTransform> HUDPAT;
		osg::ref_ptr<osg::PositionAttitudeTransform> mapPAT;
		osg::ref_ptr<osg::PositionAttitudeTransform> timesPAT;
		osg::ref_ptr<osg::PositionAttitudeTransform> P1PAT;
		osg::ref_ptr<osg::PositionAttitudeTransform> P2PAT;
		osg::ref_ptr<osg::PositionAttitudeTransform> tachometerArrowPAT;
		osg::ref_ptr<osg::PositionAttitudeTransform> speedometerArrowPAT;
		osgText::Text* HUDGear;
		osgText::Text* HUDInfoCenter;
		osg::Timer_t HUDInfoCenterTO;
		osg::Timer_t HUDInfoCenterStart;
		osg::Timer_t HUDSectorTimeTO;
		osg::Timer_t HUDSectorTimeStart;
		osg::Timer_t HUDRecordSectorTimeTO;
		osg::Timer_t HUDRecordSectorTimeStart;
		float MAX_RPM;
		float MAX_SPEED;
		int screen_width, screen_height;
		float scale,map_scale;
		int p_num;
		int x_offset;

	public:
		osgText::Text* HUDLapCnt;
		osgText::Text* HUDBestLap;
		osgText::Text* HUDBestLapS1;
		osgText::Text* HUDBestLapS2;
		osgText::Text* HUDLastLap;
		osgText::Text* HUDRecord;
		osgText::Text* HUDRecordS1;
		osgText::Text* HUDRecordS2;
		osgText::Text* HUDRecordman;
		osgText::Text* HUDactTime;
		osg::Geode* sector_timeP;
		osg::Geode* sector_timeN;
		osg::Geode* Recordsector_timeP;
		osg::Geode* Recordsector_timeN;

		HUD(const int width, const int height, const float max_rpm, const float max_speed, int player, osg::Vec3d p1col, osg::Vec3d p2col = osg::Vec3d(0,0,0), const std::string gauge_img = "data/HUD/gauge.png", const std::string map_img = "data/Brno/map.png");

		osg::Projection* getHUD();

		void updateTachometer(VehicleHUDData data);

		void updateMap(osg::Vec4d pos);
		void updateMap();

		void setMainInfoText(std::string text, int timeout = 3000, float font_size = 40, osg::Vec4 color = osg::Vec4(1,1,1,1));

		void setSectorTime(std::string text, int timeout = 3000, bool positive = true);

		void setRecordSectorTime(std::string text, int timeout = 3000, bool positive = true);

		void resetMainInfoText();
};

#endif