/*
	Physical Simulation in Graphics Scene - master's thesis
    Copyright (C) 2010-2011  Marin Javorka, FIT VUT in Brno, http://www.fit.vutbr.cz, marian.javorka@deq.sk

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
*/
#include "menu.h"

MENU::MENU(const int width, const int height){

	// MENU geometry node
	MENUGeode = new osg::Group();
	// MENU projection node
	MENUProjectionMatrix = new osg::Projection();
	// MENU PAT
	MENUPAT = new osg::PositionAttitudeTransform();

	// set up orthogonal projection
	MENUProjectionMatrix->setMatrix(osg::Matrix::ortho2D(0,width,0,height));

	// set identity as model view matrix
	osg::MatrixTransform* MENUModelViewMatrix = new osg::MatrixTransform;
	MENUModelViewMatrix->setMatrix(osg::Matrix::identity());

	// make sure the model view matrix is not affected by transforms applied above in scene graph
	MENUModelViewMatrix->setReferenceFrame(osg::Transform::ABSOLUTE_RF);

	// add model view matrix as child of the projection matrix
	MENUProjectionMatrix->addChild(MENUModelViewMatrix);
	// add geode as child of model view matrix
	MENUModelViewMatrix->addChild(MENUGeode);

	osg::StateSet* stateSet = new osg::StateSet();

	// disable depth_test
	stateSet->setMode(GL_DEPTH_TEST, osg::StateAttribute::OFF);
	
	stateSet->setRenderBinDetails( 11, "RenderBin");

	// set StateSet
	MENUGeode->setStateSet(stateSet);

	osg::Geode* drawables = new osg::Geode;
	MENUPAT->addChild(drawables);


	// Headline
	osgText::Text* headline = new osgText::Text();
	headline->setCharacterSize(100);
	headline->setFont("data/font/AlphaFridgeMagnets.ttf");
	headline->setText("Racing 2011");
	headline->setAxisAlignment(osgText::Text::SCREEN);
	headline->setAlignment(osgText::Text::CENTER_TOP);
	headline->setPosition( osg::Vec3(width/2,height-100,-1) );
	headline->setColor( osg::Vec4(1, 1, 1, 0.9) );
	headline->setFontResolution(512,512);
	headline->setBackdropType(osgText::Text::DROP_SHADOW_BOTTOM_RIGHT);

	drawables->addDrawable(headline);

	// autors
	osgText::Text* headline2 = new osgText::Text();
	headline2->setCharacterSize(20);
	headline2->setFont("data/font/AlphaFridgeMagnets.ttf");
	headline2->setText("Marin Javorka, xjavor12");
	headline2->setAxisAlignment(osgText::Text::SCREEN);
	headline2->setAlignment(osgText::Text::LEFT_TOP);
	headline2->setPosition( osg::Vec3(width/2+50,height-180,-1) );
	headline2->setColor( osg::Vec4(0.8, 0.8, 0.8, 0.9) );
	headline2->setFontResolution(512,512);
	headline2->setBackdropType(osgText::Text::DROP_SHADOW_BOTTOM_RIGHT);

	drawables->addDrawable(headline2);

	// menu items
	osgText::Text* defaultItem = new osgText::Text();
	defaultItem->setCharacterSize(50);
	defaultItem->setFont("data/font/AlphaFridgeMagnets.ttf");
	defaultItem->setAxisAlignment(osgText::Text::SCREEN);
	defaultItem->setAlignment(osgText::Text::LEFT_TOP);
	defaultItem->setColor( osg::Vec4(1, 1, 1, 0.9) );
	defaultItem->setFontResolution(512,512);
	defaultItem->setBackdropType(osgText::Text::OUTLINE);

	menuItems.push_back(new osgText::Text(*defaultItem));
	menuItems.at(0)->setText("Play drag race");
	menuItems.at(0)->setPosition( osg::Vec3(50,300,-1) );
	menuItems.at(0)->setColor( osg::Vec4(1, 0, 0, 0.9) );
	drawables->addDrawable(menuItems.at(0));
	
	menuItems.push_back(new osgText::Text(*defaultItem));
	menuItems.at(1)->setText("Free ride");
	menuItems.at(1)->setPosition( osg::Vec3(50,250,-1) );
	drawables->addDrawable(menuItems.at(1));
	
	menuItems.push_back(new osgText::Text(*defaultItem));
	menuItems.at(2)->setText("Exit");
	menuItems.at(2)->setPosition( osg::Vec3(50,200,-1) );
	drawables->addDrawable(menuItems.at(2));



	MENUGeode->addChild(MENUPAT);
}

osg::Projection* MENU::getMENU(){
	return MENUProjectionMatrix;
}

int MENU::getMenuItemsCount(){
	return menuItems.size();	
}

void MENU::highlightMenuItem(int oldHighlighted, int newHighlighted){
	menuItems.at(oldHighlighted)->setColor(	osg::Vec4(1,1,1,0.9) );
	menuItems.at(newHighlighted)->setColor(	osg::Vec4(1,0,0,0.9) );
}