/*
	Physical Simulation in Graphics Scene - master's thesis
    Copyright (C) 2010-2011  Marin Javorka, FIT VUT in Brno, http://www.fit.vutbr.cz, marian.javorka@deq.sk

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
*/
#ifndef _PHYSICS_H
#define _PHYSICS_H

//#include <osgbBullet/MotionState.h>
//#include <osgbBullet/CollisionShapes.h>
//#include <osgbBullet/Utils.h>
#include <osg/PositionAttitudeTransform>
#include <osg/MatrixTransform>

#include <btBulletDynamicsCommon.h>
#include "BulletCollision/CollisionShapes/btShapeHull.h"
#include <iostream>
#include <fstream>

using namespace std;

class Physics{
	private:
		float g;
		btVector3 gravity;
		float temperature;
		int p;// Pa
		float R_spec;// J/(kgK)
		float airD;
	public:
		Physics();
	
		// init physics
		btDynamicsWorld* initPhysics();

		// create rigid body
		btRigidBody* createRigidBody(btDynamicsWorld* world, float mass, const btTransform& startTransform, btCollisionShape* shape, double inerX = 0, double inerY = 0, double inerZ = 0);
		btRigidBody* createRigidBody(float mass, const btTransform& startTransform, btCollisionShape* shape);

		btCollisionShape* createCollisionShapeFromBvhTriangleMeshShape(const std::string file);

		btConvexHullShape* createConvexHullShapeFromConvexTriangleMeshShape(const std::string file);

		void setTemperatureC(float temp){temperature = temp + 273.15; airD = p/(temperature*R_spec);};
		void setTemperatureK(float temp){temperature = temp; airD = p/(temperature*R_spec);};

		float getAirDensity(){return airD;};

		~Physics();
};

class RigidBodyUpdateCallback: public osg::NodeCallback
{
    private:
        btRigidBody *_body;

    public:
        RigidBodyUpdateCallback(btRigidBody *body) :
            _body(body)
        {
        }

        virtual void operator()(osg::Node* node, osg::NodeVisitor* nv)
        {
            btScalar m[16];

            btDefaultMotionState* myMotionState = (btDefaultMotionState*) _body->getMotionState();
            myMotionState->m_graphicsWorldTrans.getOpenGLMatrix(m);

            osg::Matrixf mat(m);

            osg::PositionAttitudeTransform *pat = dynamic_cast<osg::PositionAttitudeTransform *> (node);
            pat->setPosition(mat.getTrans());
            pat->setAttitude(mat.getRotate());

            //std::cout << mat.getTrans().x() << ", " << mat.getTrans().y() << ", " << mat.getTrans().z() << std::endl;

            traverse(node, nv);
        }
};

extern Physics *physics;

#endif