/*
	Physical Simulation in Graphics Scene - master's thesis
    Copyright (C) 2010-2011  Marin Javorka, FIT VUT in Brno, http://www.fit.vutbr.cz, marian.javorka@deq.sk

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
*/
#include "scene.h"

// fragment shader to change sky texture
static const char* fragSource = 
"uniform sampler2D tex0;"
"uniform sampler2D tex1;"
"uniform float alpha;"
""
"void main()"
"{"
"   vec4 t0 = texture2D(tex0, gl_TexCoord[0].st);"
"   vec4 t1 = texture2D(tex1, gl_TexCoord[0].st);"
"   gl_FragColor = mix( t0, t1, alpha);"
"}";

osg::ref_ptr<osg::LightSource> lightSource1;

osg::Group* loadTrackAndEnviroment(btDynamicsWorld* m_dynamicsWorld, int weather, osg::StateSet** stateset){

	// scenegraph root
	osg::Group* root = new osg::Group;

	/////////////////////////
	// Create track
	/////////////////////////
	
	// load track model and create PAT
	osg::ref_ptr<osg::Node> skyModel = osgDB::readNodeFile("data/Brno/sky.3ds");
    osg::ref_ptr<osg::PositionAttitudeTransform> skyPAT = new osg::PositionAttitudeTransform;

    skyPAT->addChild(skyModel.get());
	skyPAT->setNodeMask(skyPAT->getNodeMask()& ~0x2);

	osg::StateSet* stateSetLOFF = new osg::StateSet();

	// disable depth_test
	stateSetLOFF->setMode(GL_LIGHTING, osg::StateAttribute::OFF);

	skyPAT.get()->setStateSet(stateSetLOFF);

	{
		// load sky textures
		osg::Image* image = osgDB::readImageFile("data/Brno/sky.jpg"); 
		osg::Image* image2 = osgDB::readImageFile("data/Brno/sky_oc.jpg"); 

		osg::Texture2D* tex2d = new osg::Texture2D( image ); 
		tex2d->setWrap(osg::Texture2D::WRAP_S, osg::Texture2D::REPEAT); 
		tex2d->setWrap(osg::Texture2D::WRAP_T, osg::Texture2D::REPEAT);
		osg::Texture2D* tex2d2 = new osg::Texture2D( image2 );
		tex2d2->setWrap(osg::Texture2D::WRAP_S, osg::Texture2D::REPEAT); 
		tex2d2->setWrap(osg::Texture2D::WRAP_T, osg::Texture2D::REPEAT); 

		// fragment shader to use
		osg::ref_ptr<osg::Shader> f = new osg::Shader( osg::Shader::FRAGMENT );
		f->setShaderSource( fragSource );

		osg::ref_ptr<osg::Program> rPrg = new osg::Program;
		rPrg->setName( "myShader" );
		rPrg->addShader( f.get() );

		
		(*stateset) = Vehicle::findNodeByName(skyModel,"Koule")->getOrCreateStateSet();

		(*stateset)->setAttributeAndModes( rPrg.get(), osg::StateAttribute::ON );

		(*stateset)->setTextureAttributeAndModes(2,tex2d2, osg::StateAttribute::ON);
		osg::Uniform* tex1 = new osg::Uniform("tex1",2);
		(*stateset)->addUniform(tex1);

		(*stateset)->setTextureAttributeAndModes(0,tex2d, osg::StateAttribute::ON);
		osg::Uniform* tex0 = new osg::Uniform("tex0",0);
		(*stateset)->addUniform(tex0);
	}

	// add track PAT to root
    root->addChild(skyPAT.get());


	// load track model and create PAT
	osg::ref_ptr<osg::Node> trackModel;
	if( weather != 2 && weather != 4)
		trackModel = osgDB::readNodeFile("data/Brno/brno.3ds");
	else
		trackModel = osgDB::readNodeFile("data/Brno/brno_w.3ds");
    osg::ref_ptr<osg::PositionAttitudeTransform> trackPAT = new osg::PositionAttitudeTransform;

	osg::StateSet* stateSet = new osg::StateSet();

	// turn on blending
	stateSet->setMode(GL_BLEND, osg::StateAttribute::ON);

	// disable depth_test
	stateSet->setMode(GL_DEPTH_TEST, osg::StateAttribute::ON);
	stateSet->setRenderingHint(osg::StateSet::TRANSPARENT_BIN);

	//trackModel.get()->setStateSet(stateSet);

    trackPAT->addChild(trackModel.get());

	
	trackPAT->setNodeMask(trackPAT->getNodeMask()& ~0x2);

	// add track PAT to root
    root->addChild(trackPAT.get());

	// physics
    // create ground collision shape
    {
		// create collision shape
		btCollisionShape* gnd_shape = physics->createCollisionShapeFromBvhTriangleMeshShape("data/Brno/brno.x");

        btVector4 pos;
        pos.setValue(0, 0, 0,0);
        btTransform trans;
        trans.setIdentity();
        trans.setOrigin(pos);
        btScalar mass = 0.f;

		// create physical body from collision shape to physical world
        btRigidBody* gnd = physics->createRigidBody(m_dynamicsWorld, mass, trans, gnd_shape);


        gnd->setUserPointer(trackModel.get());
        gnd->setCollisionFlags(btCollisionObject::CF_STATIC_OBJECT);

        gnd->setActivationState(DISABLE_DEACTIVATION);

		gnd->setFriction(1.8f);
		gnd->setCompanionId(222333);
    }
	
	// load track model and create PAT
	osg::ref_ptr<osg::Node> detailModel = osgDB::readNodeFile("data/Brno/detail_veryHigh.3ds");
    osg::ref_ptr<osg::PositionAttitudeTransform> detailPAT = new osg::PositionAttitudeTransform;

//	bm->prepareChildren();
    detailPAT->addChild(detailModel.get());
	detailPAT->setNodeMask(detailPAT->getNodeMask()& ~0x2);

	osg::StateSet* detailStateSet = new osg::StateSet();

	// turn on blending
	detailStateSet->setMode(GL_BLEND, osg::StateAttribute::ON);

	// disable depth_test
	detailStateSet->setMode(GL_DEPTH_TEST, osg::StateAttribute::ON);
	detailStateSet->setMode(GL_LIGHTING, osg::StateAttribute::OFF);
	detailStateSet->setRenderingHint(osg::StateSet::TRANSPARENT_BIN);

	detailModel.get()->setStateSet(detailStateSet);

	// add track PAT to root
   root->addChild(detailPAT.get());

	/////////////////////////
	// Create track END
	/////////////////////////

	/////////////////////////
	// Lights
	/////////////////////////

	osg::ref_ptr<osg::Group> lightGroup (new osg::Group);
	osg::ref_ptr<osg::StateSet> lightSS (root->getOrCreateStateSet());
	lightSource1 = new osg::LightSource;
	osg::ref_ptr<osg::LightSource> lightSource2 = new osg::LightSource;
	osg::ref_ptr<osg::LightSource> lightSource3 = new osg::LightSource;

	root->addChild(lightGroup.get());

	float sun_mult = 1.0;

	if( weather == 1 )
		sun_mult = 0.8;
	else if( weather == 2 )
		sun_mult = 0.9;

	float surL = 0.6;
	float mainL = 1.0;
	
	//lightGroup.get()->setNodeMask(lightGroup.get()->getNodeMask()& ~0x2);

	// create a local light.
	{
  		osg::ref_ptr<osg::Light> myLight = new osg::Light;
		myLight->setLightNum(0);
		myLight->setPosition(osg::Vec4f(-550,-1600,2000.0,1.0f));
			myLight->setDiffuse(osg::Vec4(surL*sun_mult,surL*sun_mult,surL*sun_mult,1.0f));
			myLight->setAmbient(osg::Vec4(surL*sun_mult,surL*sun_mult,surL*sun_mult,1.0f));
			myLight->setConstantAttenuation(1.0f);
		lightSource3->setLight(myLight.get());

		lightSource3->setLocalStateSetModes(osg::StateAttribute::ON); 
		lightSource3->setStateSetModes(*lightSS,osg::StateAttribute::ON);

		lightGroup->addChild(lightSource3.get());
	}

	// create a local light.
	{
  		osg::ref_ptr<osg::Light> myLight = new osg::Light;
		myLight->setLightNum(1);
		myLight->setPosition(osg::Vec4f(2000,-200,2000.0,1.0f));
			myLight->setDiffuse(osg::Vec4(surL*sun_mult,surL*sun_mult,surL*sun_mult,1.0f));
			myLight->setAmbient(osg::Vec4(surL*sun_mult,surL*sun_mult,surL*sun_mult,1.0f));
			myLight->setConstantAttenuation(1.0f);
		lightSource2->setLight(myLight.get());

		lightSource2->setLocalStateSetModes(osg::StateAttribute::ON); 
		lightSource2->setStateSetModes(*lightSS,osg::StateAttribute::ON);

		lightGroup->addChild(lightSource2.get());
	}

	// create a local light.
	{
  		osg::ref_ptr<osg::Light> myLight = new osg::Light;
		myLight->setLightNum(2);
		myLight->setPosition(osg::Vec4f(-150,-430.0,2000.0,1.0f));
			myLight->setDiffuse(osg::Vec4(mainL*sun_mult,mainL*sun_mult,mainL*sun_mult,1.0f));
			myLight->setAmbient(osg::Vec4(mainL*sun_mult,mainL*sun_mult,mainL*sun_mult,1.0f));
			//myLight->setConstantAttenuation(1.0f);
			myLight->setSpecular(osg::Vec4(mainL*sun_mult,mainL*sun_mult,mainL*sun_mult,1.0f));
		lightSource1->setLight(myLight.get());

		lightSource1->setLocalStateSetModes(osg::StateAttribute::ON); 
		lightSource1->setStateSetModes(*lightSS,osg::StateAttribute::ON);

		lightGroup->addChild(lightSource1.get());
	}

	/////////////////////////
	// Lights END
	/////////////////////////

	return root;
};

osg::Group* enableShadows(osg::Group* scene){
	/////////////////////////
	// Shadows
	/////////////////////////

	osg::Group* root =  new osg::Group;

    osg::ref_ptr<osgShadow::ShadowedScene> shadowedScene = new osgShadow::ShadowedScene;

    shadowedScene->setReceivesShadowTraversalMask(ReceivesShadowTraversalMask);
    shadowedScene->setCastsShadowTraversalMask(CastsShadowTraversalMask);

	//osg::ref_ptr<osgShadow::LightSpacePerspectiveShadowMapCB> sm = new osgShadow::LightSpacePerspectiveShadowMapCB();
	//osg::ref_ptr<osgShadow::StandardShadowMap> sm = new osgShadow::StandardShadowMap;
	osg::ref_ptr<osgShadow::ShadowMap> sm = new osgShadow::ShadowMap;
    shadowedScene->setShadowTechnique(sm.get());

	sm->setLight(lightSource1->getLight());

	sm->setPolygonOffset(osg::Vec2(-1.0,1.0));

    int mapres = 1024;
    sm->setTextureSize(osg::Vec2s(mapres,mapres));

	shadowedScene->addChild(scene);

	root->addChild(shadowedScene);

	/////////////////////////
	// Shadows END
	/////////////////////////

	return root;
}

void addPlayers(vector<Player*> players, btDynamicsWorld* physicalWorld,osg::Group* scene, int width, int height){

	// init default values
	KeyMap P1Keys,P2Keys;

	osg::Vec3f P1Start, P2Start;

	P1Keys.throttle = osgGA::GUIEventAdapter::KEY_Up;
	P1Keys.brake = osgGA::GUIEventAdapter::KEY_Down;
	P1Keys.left = osgGA::GUIEventAdapter::KEY_Left;
	P1Keys.right = osgGA::GUIEventAdapter::KEY_Right;
	P1Keys.handbrake = osgGA::GUIEventAdapter::KEY_Space;
	P1Keys.reset = 'l';
	P1Keys.clutch = 'k';
	P1Keys.gear_up = 'j';
	P1Keys.gear_down = 'm';

	P2Keys.throttle = 'e';
	P2Keys.brake = 'd';
	P2Keys.left = 's';
	P2Keys.right = 'f';
	P2Keys.handbrake = osgGA::GUIEventAdapter::KEY_Shift_L;
	P2Keys.reset = 'r';
	P2Keys.clutch = 'w';
	P2Keys.gear_up = 'q';
	P2Keys.gear_down = 'a';

	P2Start = osg::Vec3f(-2.2,-40,41.1);
	P1Start = osg::Vec3f(3.8,-48,41.1);
	
	for(int i = 0; i < (int)players.size(); i++){
		// player's car
		VehicleModel car1Model;		// structure for models and collision shapes
		
		// set up models and col.shapes
		{
			// load chassis model
			car1Model.chassisModel = osgDB::readNodeFile(players.at(i)->path+"_body.3ds");

			osg::ref_ptr<osg::StateSet> modelss = players.at(i)->car->findNodeByName(car1Model.chassisModel,"body")->getOrCreateStateSet();

			osg::ref_ptr<osg::Material> nodematerial = new osg::Material;

			float r = players.at(i)->color.x(),g = players.at(i)->color.y(),b = players.at(i)->color.z();
			if( r < 0.02 )
				r = 0.02;
			if( g < 0.02 )
				g = 0.02;
			if( b < 0.02 )
				b = 0.02;

			nodematerial->setDiffuse(osg::Material::FRONT,osg::Vec4(r*0.8,g*0.8,b*0.8,1.0f));
			nodematerial->setAmbient(osg::Material::FRONT,osg::Vec4(r*0.1,g*0.1,b*0.1,1.0f));
			nodematerial->setSpecular(osg::Material::FRONT,osg::Vec4(r*0.1,g*0.1,b*0.1,1.0f));
			nodematerial->setShininess(osg::Material::FRONT, 120);
			//Attaching the newly defined state set object to the node state set
			modelss->setAttribute(nodematerial.get());

			osg::ref_ptr<osg::StateSet> windowss = players.at(i)->car->findNodeByName(car1Model.chassisModel,"window")->getOrCreateStateSet();

			osg::ref_ptr<osg::Material> windowmaterial = new osg::Material;

			osg::Vec4 wcolor = osg::Vec4(1.0,1.0,1.0,1.0f);
			windowmaterial->setDiffuse(osg::Material::FRONT_AND_BACK,wcolor);
			windowmaterial->setAmbient(osg::Material::FRONT_AND_BACK,wcolor);
			windowmaterial->setSpecular(osg::Material::FRONT_AND_BACK,wcolor);
			windowmaterial->setAlpha(osg::Material::FRONT_AND_BACK, 0.02f);
			windowmaterial->setShininess(osg::Material::FRONT, 120);
			//Attaching the newly defined state set object to the node state set
			windowss->setAttribute(windowmaterial.get());

			osg::ref_ptr<osg::StateSet> foliess = players.at(i)->car->findNodeByName(car1Model.chassisModel,"folie")->getOrCreateStateSet();

			osg::ref_ptr<osg::Material> foliematerial = new osg::Material;

			wcolor = osg::Vec4(0.0,0.0,0.0,1.0f);
			foliematerial->setDiffuse(osg::Material::FRONT_AND_BACK,wcolor);
			foliematerial->setAmbient(osg::Material::FRONT_AND_BACK,wcolor);
			foliematerial->setSpecular(osg::Material::FRONT_AND_BACK,wcolor);
			foliematerial->setAlpha(osg::Material::FRONT_AND_BACK, 0.1f);
			foliematerial->setShininess(osg::Material::FRONT, 120);
			//Attaching the newly defined state set object to the node state set
			foliess->setAttribute(foliematerial.get());

			// create chassis collision shape
			car1Model.chassisShape = physics->createConvexHullShapeFromConvexTriangleMeshShape(players.at(i)->path+"_collision.x");
			//car1Model.chassisShape = new btBoxShape(btVector3(1.f,2.35f, 0.7f));

			// load wheel model
			osg::Node* car1Wheel = osgDB::readNodeFile(players.at(i)->path+"_wheel.3ds");

			// create PAT for symetry
			osg::PositionAttitudeTransform* rotatePAT = new osg::PositionAttitudeTransform;
			rotatePAT->addChild(car1Wheel);
			// set PAT rotation
			rotatePAT->setAttitude(osg::Quat(3.14,osg::Vec3f(0,1,0)));

			// set up wheels
			car1Model.wheelModel[0] = car1Wheel;	// right-front
			car1Model.wheelModel[1] = rotatePAT;	// left-front
			car1Model.wheelModel[2] = rotatePAT;	// left-rear
			car1Model.wheelModel[3] = car1Wheel;	// right-rear

			// create wheel collision shape and assign to wheels
			btCollisionShape* car1WheelShape;
			if( players.at(i)->path.find("megane") != string::npos )
				car1WheelShape = new btCylinderShapeX(btVector3(0.26f, 0.310f, 0.310f));
			else
				car1WheelShape = new btCylinderShapeX(btVector3(0.269f, 0.368f, 0.368f));
			//btCollisionShape* car1WheelShape = physics->createConvexHullShapeFromConvexTriangleMeshShape(players.at(i)->path+"_wheel.x");
			car1Model.wheelShape[0] = car1WheelShape;
			car1Model.wheelShape[1] = car1WheelShape;
			car1Model.wheelShape[2] = car1WheelShape;
			car1Model.wheelShape[3] = car1WheelShape;
		}

		// create player one's car object
		if( i == 0 )
			players.at(i)->car = new Vehicle(physicalWorld, P1Start, car1Model, P1Keys, players.at(i)->manual);
		else
			players.at(i)->car = new Vehicle(physicalWorld, P2Start, car1Model, P2Keys, players.at(i)->manual);

		// load car specifications
		players.at(i)->car->configVehicle(string(players.at(i)->path+".ini").c_str());

		// assign chassis and wheel PATs to scene
		scene->addChild(players.at(i)->car->getChassisPAT().get());
		scene->addChild(players.at(i)->car->getWheelPAT(0).get());
		scene->addChild(players.at(i)->car->getWheelPAT(1).get());
		scene->addChild(players.at(i)->car->getWheelPAT(2).get());
		scene->addChild(players.at(i)->car->getWheelPAT(3).get());

		players.at(i)->car->getChassisPAT()->setNodeMask(0xffffffff);

		// create matrix transform for camera manipulation
		players.at(i)->carMT = new osg::MatrixTransform();
		players.at(i)->wheelMT = new osg::MatrixTransform();
		players.at(i)->car->getChassisPAT()->addChild(players.at(i)->carMT);
		players.at(i)->car->getWheelPAT(0)->addChild(players.at(i)->wheelMT);

		/////////////////////////
		// Vehicles END
		/////////////////////////


		/////////////////////////
		// HUDs
		/////////////////////////

		if(players.size()  == 2)		
			if( players.at(i)->path.find("megane") != string::npos )
				players.at(i)->hud = new HUD(width/players.size(), height, 6.4f, 190.5f, i, players.at(0)->color, players.at(1)->color, "data/HUD/gauge_megane.png");
			else
				players.at(i)->hud = new HUD(width/players.size(), height, 7.15f, 190.5f, i, players.at(0)->color, players.at(1)->color, "data/HUD/gauge_f430.png");
		else
			if( players.at(i)->path.find("megane") != string::npos )
				players.at(i)->hud = new HUD(width/players.size(), height, 6.4f, 190.5f, i, players.at(0)->color, osg::Vec3d(0,0,0), "data/HUD/gauge_megane.png");
			else
				players.at(i)->hud = new HUD(width/players.size(), height, 7.15f, 190.5f, i, players.at(0)->color, osg::Vec3d(0,0,0), "data/HUD/gauge_f430.png");
		
		players.at(i)->node = new osg::Group();
		players.at(i)->node_withoutHUD = new osg::Group();

		// disable shaddows
		players.at(i)->hud->getHUD()->setNodeMask(players.at(i)->hud->getHUD()->getNodeMask()^0x3);

		players.at(i)->node->addChild(players.at(i)->hud->getHUD());

		players.at(i)->node->addChild(enableShadows(scene));
		players.at(i)->node_withoutHUD->addChild(enableShadows(scene));
		//players.at(i)->node->addChild(scene);
		//players.at(i)->node_withoutHUD->addChild(scene);

		if(i == 1)
			players.at(0)->hud->updateMap();

		/////////////////////////
		// HUDs END
		////////////////////////

		if( i == 1 )
			break;
	}
}