/*
	Physical Simulation in Graphics Scene - master's thesis
    Copyright (C) 2010-2011  Marin Javorka, FIT VUT in Brno, http://www.fit.vutbr.cz, marian.javorka@deq.sk

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
*/
#ifndef SCENE_H
#define SCENE_H

#include <sstream>

#include <osgDB/ReadFile>
#include <osg/PositionAttitudeTransform>
#include <osg/TexEnvCombine>
#include <osg/Texture2D>

#include <btBulletDynamicsCommon.h>
#include "BulletDynamics/Vehicle/btRaycastVehicle.h"
#include "BulletCollision/CollisionShapes/btShapeHull.h"
#include "BulletCollision/CollisionShapes/btBvhTriangleMeshShape.h"

#include <osg/Light>
#include <osg/LightSource>  
#include <osg/StateSet>

#include <osgShadow/ShadowedScene>
#include <osgShadow/ShadowMap>
#include <osg/BlendFunc>

#include <vector>
#include <string>

#include "physics.h"
#include "my_key_event_handler.h"
#include "hud.h"
#include "vehicle.h"
#include "camera.h"

#include <osgShadow/ShadowedScene>
#include <osgShadow/StandardShadowMap>
#include <osgShadow/LightSpacePerspectiveShadowMap>

#include <osgGA/KeySwitchMatrixManipulator>


const int ReceivesShadowTraversalMask = 0x1;
const int CastsShadowTraversalMask = 0x2;


// player struct
struct Player{
	osg::Group* node;
	osg::Group* node_withoutHUD;
	Vehicle* car;
	HUD* hud;
	osg::MatrixTransform* carMT;
	osg::MatrixTransform* wheelMT;
	string path;
	osg::Timer_t lap_time;
	double bestLapT, bestS1, bestS2, lapT, sector1, sector2, sector3, d_old, d2_old, d3_old, recordLapT, recordS1, recordS2, bestLapTmp;
	osg::Vec3d oldPos;
	FollowCam* fc;
	int lap;
	osg::ref_ptr<osgGA::KeySwitchMatrixManipulator> ksw;
	string name;
	osg::Vec3d color;
	bool online;
	bool manual;
};

osg::Group* loadTrackAndEnviroment(btDynamicsWorld* m_dynamicsWorld, int weather, osg::StateSet** stateset);

osg::Group* enableShadows(osg::Group* scene);

void addPlayers(vector<Player*> players, btDynamicsWorld* physicalWorld,osg::Group* scene, int width, int height);

#endif